package rotty;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A simple demo to show off the possibilities of "Rotator". Use CRSR to move, shift+left/right to rotate and PGUP/PGDOWN to
 * zoom in and out.
 */
public class Demo extends JFrame implements KeyListener, ComponentListener {

   private boolean left=false;
   private boolean right=false;
   private boolean up=false;
   private boolean down=false;
   private boolean in=false;
   private boolean out=false;
   private boolean shift=false;
   private boolean render=true;
   private boolean resize=false;

   // overSample=true => Looks better, runs slower...
   private boolean overSample=false;

   public static void main(String[] args) throws Exception {
      Demo demo=new Demo();
      demo.run();
   }

   public void keyTyped(KeyEvent e) {
   }

   public void keyReleased(KeyEvent e) {
      keyAffected(e, false);
   }

   public void keyPressed(KeyEvent e) {
      keyAffected(e, true);
   }

   public void componentResized(ComponentEvent e) {
      resize=true;
   }

   public void componentHidden(ComponentEvent e) {
      render=false;
   }

   public void componentMoved(ComponentEvent e) {
   }

   public void componentShown(ComponentEvent e) {
      render=true;
   }

   private void keyAffected(KeyEvent e, boolean pressed) {
      switch (e.getKeyCode()) {
         case (KeyEvent.VK_SHIFT): {
            shift=pressed;
            break;
         }
         case (KeyEvent.VK_UP): {
            up=pressed;
            break;
         }
         case (KeyEvent.VK_DOWN): {
            down=pressed;
            break;
         }
         case (KeyEvent.VK_LEFT): {
            left=pressed;
            break;
         }
         case (KeyEvent.VK_RIGHT): {
            right=pressed;
            break;
         }
         case (KeyEvent.VK_PAGE_UP): {
            in=pressed;
            break;
         }
         case (KeyEvent.VK_PAGE_DOWN): {
            out=pressed;
            break;
         }
      }
   }

   private void run() throws Exception {
      setSize(256, 256);
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      setVisible(true);
      addKeyListener(this);
      addComponentListener(this);

      Rotator rot=new Rotator("map.jpg", getSize(), overSample);
      rot.setRotationPivot(128, 200);

      float xpos=1000;
      float ypos=500;
      float rota=0;
      float zoom=1;
      Graphics g=getGraphics();

      float[] addsX=new float[2];
      addsX[0]=1f;
      addsX[1]=0f;

      float[] addsY=new float[2];
      addsY[0]=0f;
      addsY[1]=1f;

      long s=System.currentTimeMillis();
      int fps=0;

      long ticks=System.currentTimeMillis();

      while (true) {

         if (resize) {
            rot=new Rotator("map.jpg", getSize(), overSample);
            rot.setRotationPivot(getSize().width>>1, getSize().height-72);
            rot.setZoom(zoom);
            resize=false;
         }

         long t=System.currentTimeMillis()-ticks;
         if (t>5) {
            // Timing fr die Bewegungen, damit sie nicht von Framerate abhngen...
            // ..durch die Ungenauigkeit von currentTimeMillis zuppelt das etwas...egal...
            float mul=(float)t/5f;

            ticks=System.currentTimeMillis()+(long)((mul-(int) mul)*5f);

            if (up) {
               ypos-=addsY[1]*mul;
               xpos-=addsY[0]*mul;
            }
            if (down) {
               ypos+=addsY[1]*mul;
               xpos+=addsY[0]*mul;
            }
            if (right) {
               if (!shift) {
                  xpos+=addsX[0]*mul;
                  ypos+=addsX[1]*mul;
               } else {
                  rota-=0.003f*mul;
                  rot.rotate(addsX, -0.003f*mul);
                  rot.rotate(addsY, -0.003f*mul);
               }
            }
            if (left) {
               if (!shift) {
                  xpos-=addsX[0]*mul;
                  ypos-=addsX[1]*mul;
               } else {
                  rota+=0.003f*mul;
                  rot.rotate(addsX, 0.003f*mul);
                  rot.rotate(addsY, 0.003f*mul);
               }
            }
            if (in) {
               zoom+=0.01f*mul;
               rot.setZoom(zoom);
            }
            if (out) {
               zoom-=0.01f*mul;
               if (zoom<0.001f) {
                  zoom=0.001f;
               }
               rot.setZoom(zoom);
            }
         }
         if (render) {
            g.drawImage(rot.createView(xpos, ypos, rota), 0, 0, null);
         }
         fps++;
         long div=System.currentTimeMillis()-s;
         if (div>1000) {
            s=System.currentTimeMillis();
            System.out.println(fps+"fps");
            fps=0;
         }
      }
   }
}
